/* kfortune.cpp
 
  This is my second application using Qt/Kde, it's a simple toy called
  kfortune.

  Copyright (C) 1997	Pieter Eendebak	(pte@ddsw.nl)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

#include <qcursor.h>

#include "includes.h"
#include "kfiledialog.h"

#include "defines.h"
#include "misc.h"
#include "dialog.h"
#include "datafile.h"
#include "kfortune.moc"

KApplication *kfortune;

/* ##########################
 * #	class KFortune      #
 * ##########################
 */

void KFortune::createMenu()
{
	int id ;

	file = new QPopupMenu();
	id = file->insertItem(i18n("&New fortune"),
			 this, SLOT(newfortune()) );
	accel->changeMenuAccel(file, id, KAccel::New );
	id = file->insertItem(i18n("&Save fortune"),
                                 this, SLOT(saveFortune()) );
	accel->changeMenuAccel(file, id, KAccel::Save );
 
	file->insertSeparator(-1);
	id = file->insertItem(i18n("Copy fortune to clipboard"), 
				fortune, SLOT(copyFortuneToClipboard()) ) ;
	accel->changeMenuAccel(file, id, KAccel::Copy );
	file->insertSeparator(-1);
	id = file->insertItem(i18n("&Quit"), this, SLOT(quitapp()) ); 	
	accel->changeMenuAccel(file, id, KAccel::Quit );

	options = new QPopupMenu();
	options->insertItem(	i18n("&Configure keys"),
				 this,
				 SLOT(configkeys()) );
	
	id = options->insertItem(	i18n("Configure &font"),
				this,
				SLOT(selectFont()) );

	id = options->insertItem(klocale->translate("Configure &database"),
				this,
				SLOT(configdatabase()) );
	id = options->insertItem(klocale->translate("&Misc Options..."),
				this, SLOT(configmisc()) );

	help = new QPopupMenu();
	id = help->insertItem(klocale->translate("&Help"), this, SLOT(helpmenu()) );
	accel->changeMenuAccel(file, id, KAccel::Help );
	id = help->insertItem( i18n("&About..."),
			this, SLOT(about()) ); 	
	menu = new KMenuBar(this, "menu");
	id = menu->insertItem( i18n("&File"), file, 5);
	id = menu->insertItem(i18n("Options"), options);
	menu->insertSeparator(-1);
	id = menu->insertItem(i18n("Help"), help);

	if (display==USE_DISPLAY) setMenu(menu);

	/* now create the mouse popup-menu */
	mouse_menu = new QPopupMenu ;
	id = mouse_menu->insertItem( i18n("New fortune"),
		this, SLOT(newfortune()) ) ; 
	id = mouse_menu->insertItem( i18n("Misc options"),
		this, SLOT(configmisc()) ) ; 
	mouse_menu->insertSeparator(-1) ;
	id = mouse_menu->insertItem( i18n("Quit"),
		this, SLOT(quitapp()) ) ; 

}

void KFortune::mousePressEvent( QMouseEvent *m )
{
	#ifdef DEBUG
	printf("KFortune:: mousePressEvent: %d\n",
		m->button() ) ;
	#endif

	QPoint p = QCursor::pos() ;
	mouse_menu->popup( p ) ;
}

void KFortune::resizeEvent( QResizeEvent * )
{
	resizeEvent();
}

void KFortune::resizeEvent( bool all )
{
	QSize s = size();
	breed=s.width();
	hoog=s.height();

	const int menu_height=30 ;
	const int button_height=30 ;

	const int left_offset=10 ;
	const int right_offset=10 ;
	int top_offset=12+menu_height;
	int bottom_offset=10+button_height ;

	if( buttons==USE_BUTTONS )
	{
		bottom_offset=10+button_height ;
	}
	else	bottom_offset=10 ;	

	if (display==USE_DISPLAY )
 	{
		top_offset=10;
		menu->hide();
	}
	else if ( display>=USE_DISPLAY_ONCE)
	{
		display--;
		menu->hide() ;

		if(display==NO_DISPLAY)
		{
			top_offset=30;
			setMenu(menu);
			menu->show();
			all=true;
		}
	}
	else
	{
		top_offset=30 ;
		menu->show() ;
	}
	
	int bbreed=breed-left_offset-right_offset ;
	int bhoog=hoog-top_offset-bottom_offset;

	if (all)
	{
		menu->setGeometry(0,0, s.width(), menu_height );
		box->setGeometry(left_offset, top_offset, bbreed, bhoog);

		fortune->setGeometry(11,16,bbreed-20,bhoog-22);
		
		quit->setGeometry( breed-200,hoog-40,90,button_height);
		next->setGeometry( breed-100 ,hoog-40,90,button_height);
	}

	fortune->show();
}

void KFortune::configmisc()
{
	this->killTimers() ;

	Misc *m;
	m = new Misc(launch, offensive, autoexit,
			autoexit_time, choosing, display,
			buttons,
			this, 0); 
	if (m->exec() )
	{
		if ( m->display()==USE_DISPLAY )
			setDisplay( USE_DISPLAY ) ;
		else	setDisplay( NO_DISPLAY ) ;
		if ( m->offensive()==USE_OFFENSIVE )
			setOffensive( USE_OFFENSIVE ) ;
		else	setOffensive( NO_OFFENSIVE ) ;

		if ( m->offensive()==USE_OFFENSIVE )
			setOffensive( USE_OFFENSIVE ) ;
		else	setOffensive( NO_OFFENSIVE ) ;

		if (m->choosing()==CHOOSING_NORMAL )
			choosing=CHOOSING_NORMAL;
		else choosing=CHOOSING_SIMPLE;

		if ( m->launch()==USE_AUTOLAUNCH )
			setLaunch( USE_AUTOLAUNCH ) ;
		else	setLaunch( NO_AUTOLAUNCH ) ;

		if ( m->buttons()==USE_BUTTONS )
			setButtons( USE_BUTTONS ) ;
		else	setButtons( NO_BUTTONS ) ;
		
		if(m->autoexit()==DO_COUNTDOWN )
		{
			autoexit = DO_COUNTDOWN ;
			autoexit_time = m->tijd ;
		}
		else	autoexit = NO_COUNTDOWN ;
	}
	
	rereadData();
}

void KFortune::setDisplay( int d )
{
	display=d ;
	resizeEvent( true ) ;
}

void KFortune::setOffensive( int a)
{
	QString tmp;

	offensive=a ;

	if ( offensive==USE_OFFENSIVE )
	{
		config->setGroup("DataBase");

		for(int i=0;i<databases;i++)
		{
			tmp.sprintf("%d%s", i, "off");
			if ( config->readNumEntry( tmp.data() )==1 )
			{
				tmp.sprintf("%d%s", i, "enabled");
				config->writeEntry( tmp.data(), 0 );
			}
		}

	}
}

void KFortune::setButtons( int b )
{
	buttons=b;

	if( b==USE_BUTTONS )
	{
		quit->show() ;
		next->show() ;
	}
	else
	{
		quit->hide() ;
		next->hide() ;
	}

	resizeEvent( true ) ;
}

void KFortune::setLaunch( int a )
{
	#ifdef DEBUG
	printf("Setting autolaunch to: %s\n",
		a==USE_AUTOLAUNCH? "on":"off" ) ;
	#endif

	char *home = getenv("HOME") ;

	QString tmp ;
	tmp.sprintf("%s%s", home, "/Desktop/Autostart/" );
	QString tmp2 = kfortune->kde_datadir();
	tmp2.append("/kfortune/");

	tmp.append("kfortune.kdelnk");
	tmp2.append("kfortune.kdelnk");

	launch = a ;
	if ( launch==USE_AUTOLAUNCH )
		copyFile( tmp2.data(), tmp.data() );
	else	remove( tmp.data() );
}

void KFortune::configdatabase()
{
	DataBase *d;
	d = new DataBase( offensive, config, this, 0);
	if (d->exec() )
	{
		// get all the options...
		// and configure database...
		config->setGroup("DataBase");

		QString p, q;	
		int r;	
	
		for(int i=0;i<databases;i++)
		{
			p.setNum(i);
			p.append("enabled");
			
			if (d->rows[i]->isChecked() ) r=1;
			else r=0;

			config->writeEntry( p.data(), r);
		}
	}

	config->sync();

	// now re-load the data!
	rereadData();

	delete d;
}

void KFortune::initKeys()
{
	accel->insertItem( i18n("About"), "About", "CTRL+A");
	accel->connectItem( "About", this, SLOT(about()) ) ;
	accel->connectItem( KAccel::Help, this, SLOT(helpmenu()) );
	accel->connectItem( KAccel::Quit, this, SLOT( quitapp()) );
	accel->insertItem( i18n("New fortune"),"New fortune", "CTRL+N" );
	accel->connectItem( "New fortune", this, SLOT(newfortune()) ) ;
	accel->insertStdItem( KAccel::Save, i18n("Save fortune") ) ;
	accel->connectItem( KAccel::Save, this, SLOT(saveFortune()) ) ;
	accel->insertItem( i18n("Copy to clipboard"),"Copy to clipboard",
			"CTRL+C" );
 	accel->connectItem( "Copy to clipboard", fortune, 	
			SLOT(copyFortuneToClipboard() ) ) ;

	accel->insertItem( i18n("Configure kfortune"),"configure",
		"CTRL+O" );
	accel->connectItem( "configure", this, SLOT(configmisc()) ) ;

	accel->readSettings(config) ;
}

void KFortune::cacheUpdate()
{
	// here we update the cache
	// (call updateCache() for each member of database[]
	QString tmp, tmp2, tmp3;
	DataFile h;
	int i;
	
	config->setGroup("DataBase");
	databases=config->readNumEntry("database");

	for(i=0;i<databases;i++)
	{
		tmp.sprintf("%d", i);
		tmp2.sprintf("%s", (config->readEntry("datadir")).data() );
		tmp3.sprintf("%s", 
				(config->readEntry(tmp.data())).data() );
		h.setName( tmp2.data(), tmp3.data() );
		h.updateCache();
	}
}

void KFortune::newfortune()
{
	QString tmp;
	int base=0;

	/* try to pick a random fortune */
	/* pick database */

	if ( choosing==CHOOSING_NORMAL )
	{
		int j = (int)( enabled*getRandom() ) ;

		while(true)
		{
		if ( (int)( total*getRandom() ) 
				 < a[j] )
			{
				base=j;
				break;
			}
			j==enabled-1? j=0: j++;
		}

	}
	else	/* CHOOSING_SIMPLE */
		base = (int)( enabled * getRandom() );

	int x = database[base]->getNumber();
	int number = (int)( x*getRandom() );

	if ( (lastdbase==base) && (lastnumber==number) )
	{
		QString tmp2 = i18n("The program tried to"
			"display the same fortune twice!\n\n"
			"The chance that this would happen with your current"
			"database is");
		tmp.sprintf("%s 1/%d.", tmp2.data(), total );
		
		setFortune( tmp ) ;
		lastnumber=-1;
		return;
	}
	
	lastdbase=base;
	lastnumber=number;
	
	/* get the fortune text */
	tmp = database[base]->getFortune(number);
	/* Set the fortune text */
	setFortune ( tmp );
}

void KFortune::saveFortune()
{
	/* get the filename */
	QString file = KFileDialog::getOpenFileName();
	QString fort = fortune->getFortuneText() ; 

	if ( ! file.isEmpty() )
	{
		/* save the fortune */
		QFile f;
		f.setName( file.data() );
		if ( f.open(IO_WriteOnly ) )
		{
			f.writeBlock( fort.data(), fort.length() );
			f.close();

			KMsgBox::message(this, CAPTION,
				i18n(
	"The fortune has been saved.\n"
	"You can copy fortunes faster using cut and paste\n"
	"or Drag&Drop. See the online documentation for\n"
	"more details.\n"
				) ) ;
		}
	}
}

void KFortune::setFortune( QString txt )
{
	fortune->setFortuneText( txt );
	resizeEvent(false);
}

void KFortune::setDatabaseEnabled( bool b, int db )
{
	QString tmp ;
	tmp.sprintf("%d%s", db, "enabled") ;

	if( b )	config->writeEntry( tmp.data(), 1 ) ; 
	else	config->writeEntry( tmp.data(), 0 ) ; 
}

void KFortune::setDatabaseOffensive( bool b, int db )
{
	QString tmp ;
	tmp.sprintf("%d%s", db, "off") ;

	if( b )	config->writeEntry( tmp.data(), 1 ) ; 
	else	config->writeEntry( tmp.data(), 0 ) ; 
}

bool KFortune::databaseEnabled( int db )
{
	QString tmp ;
	tmp.sprintf("%d%s", db, "enabled") ;
	return config->readNumEntry( tmp.data(), 0 ) ; 
}

bool KFortune::databaseOffensive( int db ) 
{
	QString tmp ;
	tmp.sprintf("%d%s", db, "off") ;
	return config->readNumEntry( tmp.data(), 0 ) ; 
}

QString KFortune::databaseFile( int db )
{
	QString tmp ;
	tmp.sprintf("%d", db ) ;
	return config->readEntry( tmp.data(), "" ) ; 
}


void KFortune::initConfig()
{
	QString tmp;	

	config = kapp->getConfig();
	
	/* load the misc options */
	config->setGroup("Options");

	launch = config->readNumEntry("Launch", NO_AUTOLAUNCH ) ;
	offensive = config->readNumEntry("Offensive", NO_OFFENSIVE );

	if ( autoexit==CONFIG_COUNTDOWN )
	{
		autoexit = config->readNumEntry("Autoexit", NO_COUNTDOWN) ;
		autoexit_time = config->readNumEntry("Time", 10 ) ;
	}
	if ( autoexit==DO_COUNTDOWN)
		timer_id = startTimer( 1000*autoexit_time );

	choosing = config->readNumEntry("Choosing method", CHOOSING_NORMAL );

	if ( display==CONFIG_DISPLAY )
		display = config->readNumEntry("Display", NO_DISPLAY );
	if ( buttons==CONFIG_BUTTONS )
		buttons = config->readNumEntry("Buttons", USE_BUTTONS );
	setButtons( buttons ) ;

	/* set the font */
	fortunefont = config->readFontEntry("Text font", 
					&fortunefont ) ;
	fortune->setFont( fortunefont ) ;

	config->setGroup("DataBase");

	/* needed by various functions */
	tmp = kfortune->kde_datadir() ;
	tmp.append( "/kfortune/" );
	config->writeEntry("datadir", tmp.data() );	
	base.sprintf("%s", tmp.data() );

	/* if there no database yet, create it */
	if ( !config->hasKey("database") )
	{
		char tmp2[15]=LIST_OFFENSIVE;

		/* start filling the database */
		config->writeEntry("database", STD_BASE);
		config->writeEntry("0", BASE0);
		config->writeEntry("1", BASE1);
		config->writeEntry("2", BASE2);
		config->writeEntry("3", BASE3);
		config->writeEntry("4", BASE4);
		config->writeEntry("5", BASE5);
		config->writeEntry("6", BASE6);
		config->writeEntry("7", BASE7);
		config->writeEntry("8", BASE8);
		config->writeEntry("9", BASE9);
		config->writeEntry("10", BASE10);

		for(int i=0;i<STD_BASE;i++)
		{
			setDatabaseEnabled( true, i ) ;
			setDatabaseOffensive( tmp2[i]=='1', i  ) ;
		}	
		config->sync();
	}

	/* check if we are working with the correct version */

	config->setGroup("Version");
	tmp=config->readEntry("version", "none");
	if ( strcmp( tmp.data(), VERSION) )
	{
		printf("A new version of KFortune has been installed.\n");
		config->writeEntry("version", VERSION);
		cacheUpdate();
	}

	/* this is needed for the random function */
	int p = int((QTime::currentTime()).msec()*10000);
	srandom(p);
	srand(p);

}

void KFortune::rereadData()
{
	delete [] database;
	readData();
}


void KFortune::readData()
{
	int i=0;
	QString tmp;

	// just needed...
	lastdbase=-1;

	config->setGroup("DataBase");

	databases=config->readNumEntry("database");

	/* get the enabled files into the database */
	enabled=0;

	for(i=0;i<databases;i++)
	{
		if ( databaseEnabled(i) )
			enabled++;
	}	

	if (enabled==0)
	{
		setDatabaseEnabled( 1, true ) ;
		enabled=1;
	}

	database = new DataFile *[enabled];

	int j=0;
	QString tmp2, tmp3;

	for(i=0;i<databases;i++)
	{
		tmp2 = base ;
		tmp3 = databaseFile( i ) ;
		if (  databaseEnabled(i) )
 			database[j++]=new DataFile(tmp2, tmp3, this); 
	}


	/* re-init some data */ 

	if ( a!=0 )
		delete [] a ;
	a = new int[enabled];

	for(i=0;i<enabled;i++)
	{
			a[i] = database[i]->getNumber();
			total+=a[i];
	}
}

void KFortune::saveConfig()
{
	QString tmp;

	// Save misc options (some of them are already saved after
	// configuring them
	config->setGroup("Options");

	config->writeEntry("Launch", launch );
	config->writeEntry("Offensive", offensive );
	config->writeEntry("Display", display);
	config->writeEntry("Buttons", buttons);
	config->writeEntry("Choosing method", choosing);
	config->writeEntry("Autoexit", autoexit ) ;
	config->writeEntry("Time", autoexit_time ) ;

	config->writeEntry( "Text font", fortunefont ) ;

	// Save key-bindings
	accel->writeSettings(config) ;

	// Save config
	config->sync();
}

void KFortune::parseOptions( int argc, char **argv)
{
	for(int i=1;i<argc;i++)
	{
		if ( !strcmp( argv[i], "--help" ) 
		    )
		{
			printf("%s %s %s\n\n", NAME, VERSION, DATE) ;
			printf("Usage:   ") ;
			printf("kfortune [ --help ] [ noautoexit ] [ display ]") ;
			printf(" [ autoexit ]\n") ;
			printf("\nThere is probably some documentation in the KDE directory.\n" ) ;
			kfortune->quit() ;
		}
		else if (*argv[i]=='-') continue;
		/* the countdown stuff is handled in initConfig() */
		else if ( !strcmp( argv[i], "autoexit" ))
		{ 
			autoexit=DO_COUNTDOWN ;
			autoexit_time=10 ;
		}
		else if (!strcmp( argv[i], "noautoexit") )
			autoexit=NO_COUNTDOWN ;
		else if (!strcmp( argv[i], "menubar") )
			display=NO_DISPLAY;
		else if (!strcmp( argv[i], "nomenubar") )
			display=USE_DISPLAY;
		else if (!strcmp( argv[i], "buttons") )
			display=USE_BUTTONS;
		else if (!strcmp( argv[i], "nobuttons") )
			display=NO_BUTTONS;
		/* else if ... other options */
	}
}

void KFortune::timerEvent( QTimerEvent *time_event ) 
{
	int id = time_event->timerId() ;

	if( id==timer_id )
	{
		this->killTimers() ;
		quitapp() ;
	} 
}

KFortune:: KFortune ( int argc, char **argv, const char* name )
	: KTMainWindow ( name ), total(0), a(0),
		display(CONFIG_DISPLAY), buttons(CONFIG_BUTTONS),
		offensive( NO_OFFENSIVE ), choosing(CHOOSING_NORMAL),
		launch( NO_AUTOLAUNCH ), 
		autoexit(CONFIG_COUNTDOWN), timer_id(0),
		lastdbase(-2),
		lastnumber(-2)
{
	/* set the size for this application */
	setCaption(CAPTION);
	setMinimumSize(400, 200);
	resize(600, 250);
	setMaximumSize(900, 600);

	accel = new KAccel(this) ;

	/* create the child widgets */
	box = new QGroupBox( klocale->translate
		( "Your fortune for today : ") , this, "box" );
	fortune = new KFortuneZone(box);
	connect( fortune, SIGNAL(notifyMousePressEvent( QMouseEvent*)),
		this, SLOT(mousePressEvent(QMouseEvent *)) );
	/* TODO: make normal methods for this, no friends ! */
	fortune->display_normal->setAlignment(AlignVCenter); 
	fortune->display_normal->setFont( fortunefont );

	quit = new QPushButton( i18n("&Quit") , this, "quit");
 	connect(quit, SIGNAL(clicked()), this, SLOT(quitapp()) );
 	next = new QPushButton( i18n("&New fortune") , this, "next");
	connect(next, SIGNAL(clicked()), this, SLOT(newfortune()) );

	createMenu();

	/* now we create the options */
	parseOptions(argc, argv);
	initConfig();
	initKeys();


	/* read in the data! */
	readData();
	newfortune();
}

KFortune::~KFortune()
{
	delete [] database;
	delete fortune;

	delete accel;
	delete config;
}

void KFortune::selectFont()
{
	KFontDialog::getFont( fortunefont );
	fortune->setFont( fortunefont);
}

void KFortune::quitapp()
{
	saveConfig() ;
	kfortune->quit();	
}

void KFortune::about()
{
	QString tmp;
	tmp.sprintf("%s%s", base.data(), "kfortune.jpg");
	
	QPixmap *icon = new QPixmap( tmp.data() );
	About *a = new About( icon, this, 0);
	a->exec();

	delete a;
	delete icon;
}


void KFortune::notImpl()
{
	KMsgBox::message(this, CAPTION,
		i18n("Sorry, this function isn't implemented yet...")  );
}

double KFortune::getRandom()
{
	#ifdef linux
		return ( (double)(random()) / (RAND_MAX+1.0) ) ;
	#elif solaris
		return ( ((double)rand()) / (RAND_MAX+1.0) ) ;
	#else
		return ( ((double)rand()) / (RAND_MAX+1.0) ) ;
	#endif
}

void KFortune::helpmenu()
{
	KApplication::getKApplication()->invokeHTMLHelp("","");
}

void KFortune::configkeys()
{
	KKeyDialog::configureKeys(accel);
}


/* ##########################
 * #	Main                #
 * ##########################
 */

int main(int argc, char **argv)
{
	kfortune = new KApplication(argc, argv, "kfortune" );
	KFortune *top;
	top = new KFortune( kfortune->argc(), kfortune->argv(), "kfortune");
	top->show();
	return kfortune->exec();
}

