/* kfortune_zone.cpp
 *
 * Pieter Eendebak <pte@ddsw.nl>
 *
 */

#include "includes.h"
#include <qclipbrd.h>
#include <qdragobject.h>
#include "defines.h"
#include "flabel.h"
#include "fhtml.h"

#include "kfortune_zone.moc"

/* ##########################
 * #    class KFortuneZone  #
 * ##########################
 */

KFortuneZone::KFortuneZone( QWidget *parent, const char *name )
                : QWidget ( parent, name ),  text(0), 
		html( false ), mouse_pressed( false ) 
{
	display_normal = new FLabel( this ) ;
	display_html = new FHtml( this ) ;

	connect( display_normal,
		SIGNAL(notifyMouseMoveEvent(QMouseEvent *)),
		SLOT(mouseMoveEvent(QMouseEvent *)) ) ;
	connect( display_normal,
		SIGNAL(notifyMousePressEvent(QMouseEvent *)),
		SLOT(mousePressEvent(QMouseEvent *)) ) ;
	connect( display_normal,
		SIGNAL(notifyMouseReleaseEvent(QMouseEvent *)),
		SLOT(mouseReleaseEvent(QMouseEvent *)) ) ;
	connect( display_html,
		SIGNAL(notifyMouseMoveEvent(QMouseEvent *)),
		SLOT(mouseMoveEvent(QMouseEvent *)) ) ;
	connect( display_html,
		SIGNAL(notifyMousePressEvent(QMouseEvent *)),
		SLOT(mousePressEvent(QMouseEvent *)) ) ;
	connect( display_html,
		SIGNAL(notifyMouseReleaseEvent(QMouseEvent *)),
		SLOT(mouseReleaseEvent(QMouseEvent *)) ) ;

	clipboard = kapp->clipboard() ;
	resizeEvent() ;
}


KFortuneZone::~KFortuneZone()
{
}

void KFortuneZone::copyFortuneToClipboard()
{
	//debug("copyFortuneToClipBoard") ;

	if( clipboard!=0 )
	{
		clipboard->setText( text.data() ) ;
	}
}

void KFortuneZone::mouseReleaseEvent( QMouseEvent *m )
{
	if ( (m->button()&LeftButton)!=0 )
		mouse_pressed = false ;
}

void KFortuneZone::mouseMoveEvent( QMouseEvent *m )
{
	if( mouse_pressed )
	{
		QTextDrag *drag_object = new QTextDrag( this, 0 ) ;

		drag_object->setText( text.data() ) ;
		drag_object->setPixmap( kapp->getIcon(),
					QPoint(5,5) ) ;

		drag_object->drag() ;
	}
}

void KFortuneZone::mousePressEvent( QMouseEvent *m )
{
	#ifdef DEBUG
	printf("KFortuneZone: mousePressEvent: %d\n",
		m->button() ) ;
	#endif

	if ( (m->button()&LeftButton)!=0 )
		mouse_pressed = true ;
	else if ( m->button()!=NoButton )
		emit notifyMousePressEvent( m ) ;
}

void KFortuneZone::leaveEvent( QEvent * )
{
	mouse_pressed = false ;
}

void KFortuneZone::setFortuneText( QString s )
{
	text = QString( s ) ;

	display_normal->setText( s.data() ) ;
	/* this is a memory leak, but KHtmlWidget is complicated... */
	display_html->setHtml( s.data() );

	resizeEvent() ;
}

QString KFortuneZone::getFortuneText()
{
	return text ;
}

void KFortuneZone::setFont( QFont &t )
{
	// TODO: implement
	display_normal->setFont( t ) ; 
}

void KFortuneZone::resizeEvent( QResizeEvent * )
{
	resizeEvent() ;
}


void KFortuneZone::resizeEvent()
{
	QSize s = size() ;
	QSize check = display_normal->sizeHint();

	display_normal->resize( s ) ;
	display_html->resize( s ) ;

	if (	(	(text.contains( '<' )>0) &&
			(text.contains('>')>0)
		) ||
		(check.width()>s.width() ) ||
		(check.height()>s.height())
	   )
	{
		//debug("html");
		// we assume the fortune is a html fortune
		display_normal->hide() ;
		display_html->show() ;
	}
	else
	{
		//debug("no html");
		display_normal->show() ;
		display_html->hide() ;
	}
}



	
